using System;
using System.Collections;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Web;
using Microsoft.SharePoint;
using Telerik.Web.UI.Widgets;
using System.Collections.Generic;

namespace Telerik.SharePoint
{
    internal class SPContentProvider : FileBrowserContentProvider
    {
        public override bool CanCreateDirectory
        {
            get
            {
                return true;
            }
        }

        public SPContentProvider(HttpContext context, string[] searchPatterns, string[] viewPaths, string[] uploadPaths, string[] deletePaths, string selectedUrl, string selectedItemTag)
            : base(context, searchPatterns, viewPaths, uploadPaths, deletePaths, selectedUrl, selectedItemTag)
        {

        }

        public override DirectoryItem ResolveRootDirectoryAsTree(string path)
        {
            return ResolveDirectory(path);
        }

        public override DirectoryItem ResolveDirectory(string path)
        {
            SPFolder folder = GetSPFolderFromPath(path);

            if (folder != null)
            {
                DirectoryItem dir = CreateDirectoryItem(folder);
                //properly dispose the SPWeb object
                SPWeb web = folder.ParentWeb;
                web.Dispose();
                return dir;
            }
            else
            {
                return EmptyDirectoryItem();
            }
        }

        public override string GetFileName(string url)
        {
            return GetSPFileFromPath(url).Name;
        }

        public override string GetPath(string url)
        {
            url = url.TrimEnd('/');
            string res = url.Substring(0, url.LastIndexOf("/") + 1);
            return res;
        }

        public override Stream GetFile(string url)
        {
            SPFile file = GetSPFileFromPath(url);
            if (!file.Exists)
            {
                return null;
            }
            return new MemoryStream(file.OpenBinary());
        }

        public override string StoreBitmap(Bitmap bitmap, string url, ImageFormat format)
        {
            // Get folder
            SPFolder folder = GetSPFolderFromPath(url.Substring(0, url.LastIndexOf("/")));

            if (folder != null)
            {
                // Save bitmap to memory
                MemoryStream memStream = new MemoryStream();
                bitmap.Save(memStream, ImageFormat.Bmp);

                // Get file from memory and save it
                StoreFileInSP(folder, url, memStream.GetBuffer());
            }
            return string.Empty;
        }

        public override string StoreFile(Telerik.Web.UI.UploadedFile file, string path, string name, params string[] arguments)
        {
            return this.StoreFileInternal(file.InputStream, path, name, arguments);
        }

        private string StoreFileInternal(Stream inputStream, string path, string name, params string[] arguments)
        {
            Stream fstream = inputStream;
            byte[] content = new byte[fstream.Length];

            fstream.Read(content, 0, (int)fstream.Length);
            fstream.Close();

            SPFolder folder = GetSPFolderFromPath(path.Substring(0, path.LastIndexOf("/")));

            if (folder != null)
            {
                StoreFileInSP(folder, name, content);
                return path + name;
            }
            else
            {
                return string.Empty;
            }
        }

        public override string DeleteFile(string path)
        {
            string folderPath = GetPath(path);
            SPFolder folder = GetSPFolderFromPath(folderPath);

            if (folder != null)
            {
                SPWeb web = folder.ParentWeb;
                web.AllowUnsafeUpdates = true;
                folder.Files.Delete(path);
                web.Dispose();
            }
            return string.Empty;
        }

        public override string DeleteDirectory(string path)
        {
            SPFolder folder = GetSPFolderFromPath(path);

            if (folder != null)
            {
                SPWeb web = folder.ParentWeb;
                web.AllowUnsafeUpdates = true;
                folder.ParentFolder.SubFolders.Delete(path.TrimEnd('/'));
                web.Dispose();
            }
            return string.Empty;
        }

        public override string CreateDirectory(string path, string name)
        {
            path = path.TrimEnd('/');
            SPFolder folder = GetSPFolderFromPath(path);

            if (folder != null)
            {
                SPWeb web = folder.ParentWeb;
                web.AllowUnsafeUpdates = true;
                folder.SubFolders.Add(path + "/" + name);
                web.Dispose();
            }

            return string.Empty;
        }

        public override bool CheckWritePermissions(string folderPath)
        {
            //permissions checking offloaded to SharePoint security
            SPFolder folder = GetSPFolderFromPath(folderPath);
            bool retVal = true;
            if (folder != null)
            {
                if ((GetPermissions(folder) & PathPermissions.Upload) == 0)
                {
                    retVal = false;
                }
            }
            //properly dispose the SPWeb object
            SPWeb web = folder.ParentWeb;
            web.Dispose();
            return retVal;
        }

        public override string MoveDirectory(string path, string newPath)
        {
            string message = "FolderNotFound";
            SPFolder folder = GetSPFolderFromPath(path);
            if (folder != null && folder.Exists)
            {
                SPWeb web = folder.ParentWeb;
                web.AllowUnsafeUpdates = true;
                try
                {
                    folder.MoveTo(newPath);
                    message = string.Empty;
                }
                catch (Exception)
                {
                    message = "NewFolderAlreadyExists";
                }
                web.Dispose();

            }
            return message;
        }

        public override string MoveFile(string path, string newPath)
        {
            SPFile file = GetSPFileFromPath(path);
            string message = "FileNotFound";
            if (file != null && file.Exists)
            {
                try
                {
                    SPWeb web = file.ParentFolder.ParentWeb;
                    web.AllowUnsafeUpdates = true;
                    file.MoveTo(newPath, false);
                    message = string.Empty;
                }
                catch (Exception)
                {
                    message = "NewFileAlreadyExists";
                }
            }
            return message;
        }

        private FileItem[] GetFiles(SPFolder folder)
        {
            ArrayList fileItems = new ArrayList();
            foreach (SPFile file in folder.Files)
            {
                if (IsExtensionAllowed(file.Name))
                {
                    FileItem item = CreateFileItem(file);
                    fileItems.Add(item);
                }
            }
            return (FileItem[])fileItems.ToArray(typeof(FileItem));
        }

        private FileItem CreateFileItem(SPFile file)
        {
            return new FileItem(
                file.Name,
                Path.GetExtension(file.Name),
                file.Length,
                string.Empty,
                file.ServerRelativeUrl,
                file.ServerRelativeUrl,
                GetPermissions(file)
                );
        }

        private DirectoryItem CreateDirectoryItem(SPFolder folder)
        {
            return new DirectoryItem(
                folder.Name,
                folder.ServerRelativeUrl + "/",
                folder.ServerRelativeUrl + "/",
                string.Empty,
                GetPermissions(folder),
                GetFiles(folder),
                GetFolders(folder)
                );
        }

        private DirectoryItem[] GetFolders(SPFolder folder)
        {
            List<DirectoryItem> subFolders = new List<DirectoryItem>();
            foreach (SPFolder subFolder in folder.SubFolders)
            {
                if (IsNotSystemFolder(subFolder))
                {
                    subFolders.Add(new DirectoryItem(subFolder.Name, subFolder.ServerRelativeUrl + "/", subFolder.ServerRelativeUrl + "/", string.Empty, GetPermissions(subFolder), new FileItem[0], new DirectoryItem[0]));
                }
            }
            return subFolders.ToArray();
        }

        private bool IsExtensionAllowed(string fileName)
        {
            string extension = "*" + Path.GetExtension(fileName);
            return Array.IndexOf(SearchPatterns, "*.*") >= 0 || Array.IndexOf(SearchPatterns, extension.ToLower()) >= 0;
        }

        private PathPermissions GetPermissions(SPFile file)
        {
            return GetPermissions(file.ParentFolder);
        }

        private PathPermissions GetPermissions(SPFolder folder)
        {
            return GetPermissions(folder.ParentListId, folder.ParentWeb);
        }

        private PathPermissions GetPermissions(Guid listId, SPWeb web)
        {
            SPList list = web.Lists[listId];
            PathPermissions permissions = PathPermissions.Read;

            if (list.DoesUserHavePermissions(SPBasePermissions.AddListItems))
            {
                permissions |= PathPermissions.Upload;
            }
            if (list.DoesUserHavePermissions(SPBasePermissions.DeleteListItems))
            {
                permissions |= PathPermissions.Delete;
            }
            return permissions;
        }

        private SPFolder GetSPFolderFromPath(string path)
        {
            //the path is composed of two parts
            //1. web
            //2. list and folder within the web
            SPFolder folder;
            SPWeb web;
            try
            {
                if (path.StartsWith("/"))
                {
                    //LIBRARY WITH FULL PATH
                    SPSite site = SPContext.Current.Site;
                    if (path.StartsWith(site.ServerRelativeUrl))
                        path = path.Remove(0, site.ServerRelativeUrl.Length);

                    //remove trailing slash
                    if (path.EndsWith("/"))
                        path = path.Remove(path.Length - 1);
                    string[] pathParts = path.Split(new char[] { '/' }, StringSplitOptions.RemoveEmptyEntries);

                    //open the default web in the site collection
                    web = site.OpenWeb();
                    string webPath = web.ServerRelativeUrl;
                    if (!webPath.EndsWith("/"))
                        webPath += "/";
                    int i = 0;

                    //find the web
                    while (web.Exists && i < pathParts.Length)
                    {
                        webPath += pathParts[i++] + "/";
                        if (IsNotSystemFolder(pathParts[i - 1]))
                        {
                            web.Dispose();
                            web = site.OpenWeb(webPath);
                        }
                        else
                        {
                            //path contains a reserved part - return "not found"
                            return null;
                        }
                    }
                    web.Dispose();

                    //i marks the last known web
                    i = i - 1;
                    //remove the last part and the / after it
                    webPath = webPath.Remove(webPath.Length - 1 - pathParts[i].Length);
                    web = site.OpenWeb(webPath);
                    web.AllowUnsafeUpdates = true;

                    //find the list and folder
                    webPath = "";
                    for (int j = i; j < pathParts.Length; j++)
                        webPath += pathParts[j] + "/";
                    //remove trailing slash
                    if (webPath.EndsWith("/"))
                        webPath = webPath.Remove(webPath.Length - 1);
                    //try to open folder
                    folder = web.GetFolder(webPath);
                    //if not using the default SPContext, you must dispose the site here
                    //site.Dispose();
                }
                else
                {
                    //LOCAL LIBRARY
                    web = SPContext.Current.Web;
                    web.AllowUnsafeUpdates = true;
                    folder = web.GetFolder(path);
                }

                //clean-up
                if (folder != null && !folder.Exists)
                {
                    folder = null;
                    if (web != null)
                        web.Dispose();
                }

                return folder;
            }
            catch (Exception)
            {
                return null;
            }
        }

        private SPFile GetSPFileFromPath(string path)
        {
            SPWeb web = SPContext.Current.Web;
            // This property prevents a security exception
            web.AllowUnsafeUpdates = true;

            SPFile file = null;
            while (file == null && web != null)
            {
                try
                {
                    // This property prevents a security exception
                    web.AllowUnsafeUpdates = true;
                    file = web.GetFile(path);
                }
                catch (ArgumentException)
                {
                    //argument exception means we could not find the file
                }
                finally
                {
                    web = web.ParentWeb;
                }
            }
            return file;
        }

        private DirectoryItem EmptyDirectoryItem()
        {
            return new DirectoryItem(
                "NOT FOUND",
                string.Empty,
                string.Empty,
                string.Empty,
                PathPermissions.Read,
                new FileItem[] { },
                new DirectoryItem[] { }
                );
        }

        private void StoreFileInSP(SPFolder folder, string name, byte[] content)
        {
            SPWeb web = folder.ParentWeb;
            web.AllowUnsafeUpdates = true;
            try
            {
                //file already exists
                SPFile existingFile = folder.Files[name];
                existingFile.CheckOut();
                existingFile.SaveBinary(content);
                existingFile.CheckIn("updated by RadEditor");
            }
            catch (ArgumentException)
            {
                //file does not exist
                folder.Files.Add(name, content);
            }
            PublishFileChanges(folder, name);
            web.Dispose();
        }

        private void PublishFileChanges(SPFolder folder, string name)
        {
            SPList parentList = folder.ParentWeb.Lists[folder.ParentListId];
            try
            {
                //check in, approve, and publish file
                if (folder.Files[name].CheckOutStatus != SPFile.SPCheckOutStatus.None)
                    folder.Files[name].CheckIn("updated by RadEditor");
                if (parentList.EnableModeration)
                    folder.Files[name].Approve("aproved by RadEditor");
                else if (parentList.EnableMinorVersions)
                    folder.Files[name].Publish("published by RadEditor");
            }
            catch (Exception)
            {
            }
        }
        private static string[] systemFolders = { "my pages", "forms", "_t", "_w", "lt", "wt", "wp", "masterpage", "_catalogs" };
        public static bool IsNotSystemFolder(SPFolder folder)
        {
            return IsNotSystemFolder(folder.Name);
        }
        public static bool IsNotSystemFolder(string folder)
        {
            string lowerCaseFolderName = folder.ToLower();
            return Array.IndexOf(systemFolders, lowerCaseFolderName) < 0;
        }
    }
}